// Fragment Shader  file "instancing.frag"

#version 140

precision highp float; // needed only for version 1.30


in vec2 ex_TexCoord0;
in vec4 projcord;

in  vec3 ex_Normal;

in vec3 fragmentNormal;
in vec3 fragmentPos;
in vec3 lightDir;
in vec3 lightPos;

out vec4 out_Color;

uniform sampler2D sTex0;
uniform sampler2D spectex;
uniform sampler2D shadowmap;
uniform vec3 eyepos;


float VSM(float distance, vec2 texcoords){


	float distance1 = texture(shadowmap, texcoords).r;
	float distance2 = distance1*distance1;
	
	float m = distance1;
	float s2= distance2 -  m*m;

	if(s2 < 0.0004){
		s2 = 0.0004;
	}

	if(distance <= distance1){
		return 1.0;
	}else{
		float pd = s2/(s2+(distance-m)*(distance-m));
		return pd;
	}
}

void main(void)
{
	float diffuseIntensity;
	float specular = 0.0;
	float specularIntensity; 

	vec3 light;
	vec3 normal;

	light = normalize(lightPos-normalize(fragmentPos.xyz));

	normal = normalize(fragmentNormal);
	
	diffuseIntensity = max(dot(normal, light), 0.0);

	specularIntensity = texture(spectex, ex_TexCoord0).r;
	
	if(specularIntensity > 0.0){
		vec3 eye = normalize(eyepos - fragmentPos.xyz);
		vec3 halfWay = normalize(light + eye);
		float spec = max(0.0, dot(normal, halfWay));
		specular = pow(spec, 50.0);
		specular*=specularIntensity;
    	}
  
     


	vec2 projected = vec2((projcord.x/projcord.w)*0.5+0.5,(projcord.y/projcord.w)*0.5+0.5);
	float depth = projcord.z/projcord.w-0.0025;

	if(texture(shadowmap, projected).r < depth ){
		//in shadow
		vec4 color =texture(sTex0, ex_TexCoord0)*0.2+ texture(sTex0, ex_TexCoord0)*diffuseIntensity + vec4(1.0, 1.0,1.0, 1.0)*specular;
		if(diffuseIntensity> 0.1){
			color*=0.4;
		}
		out_Color = color;
	}
	else{
		out_Color =texture(sTex0, ex_TexCoord0)*0.2+ texture2D(sTex0, ex_TexCoord0)*diffuseIntensity + vec4(1.0, 1.0,1.0, 1.0)*specular;
	}

/*
	vec4 color = texture(sTex0, ex_TexCoord0)*0.2+ texture2D(sTex0, ex_TexCoord0)*diffuseIntensity + vec4(1.0, 1.0,1.0, 1.0)*specular;
	if(diffuseIntensity> 0.1)	
		//color*=clamp(VSM(depth, projected)+0.4,0.0, 1.0 );	
		color*=VSM(depth, projected);	


	out_Color = color;
*/

}
