// Fragment Shader  file "terrain.frag"

#version 130

precision highp float; // needed only for version 1.30

in	vec2 ex_TexCoord0;
in vec4 projcord;

in  vec3 ex_Normal;

in vec3 fragmentNormal;
in vec4 fragmentPos;
in vec3 lightDir;
in vec3 lightPos;

out vec4 out_Color;

uniform sampler2D sTex0;
uniform sampler2D sTex1;
uniform sampler2D sTex2;
uniform sampler2D sTex3;
uniform sampler2D sFilter;

uniform sampler2D shadowmap;
uniform vec3 eyepos;
uniform float specularIntensity; 


float VSM(float distance, vec2 texcoords){


	float distance1 = texture(shadowmap, texcoords).r;
	float distance2 = distance1*distance1;
	
	float m = distance1;
	float s2= distance2 -  m*m;

	if(s2 < 0.000007){
		s2 = 0.000007;
	}

	if(distance <= distance1){
		return 1.0;
	}else{
		float pd = s2/(s2+(distance-m)*(distance-m));
		return pd;
	}
}


void main(void)
{
	float diffuseIntensity;
	float specular = 0.0;

	vec3 light;
	vec3 normal;

	light = normalize(lightPos-normalize(fragmentPos.xyz));

	normal = normalize(fragmentNormal);
	
	diffuseIntensity = max(dot(normal, light), 0.0);
	
	if(specularIntensity > 0.0){
		vec3 eye = normalize(eyepos - fragmentPos.xyz);
		vec3 halfWay = normalize(light + eye);
		float spec = max(0.0, dot(normal, halfWay));
		specular = pow(spec, 50.0);
		specular*=specularIntensity;
    }
  
	vec2 projected = vec2((projcord.x/projcord.w)*0.5+0.5,(projcord.y/projcord.w)*0.5+0.5);
	float depth = projcord.z/projcord.w;

	
	
	vec4 texturecolor1 = texture(sTex0, ex_TexCoord0);
	vec4 texturecolor2 = texture(sTex1, ex_TexCoord0);
	vec4 texturecolor3 = texture(sTex2, ex_TexCoord0);
	vec4 texturecolor4 = texture(sTex3, ex_TexCoord0);
	
	vec2 filtercoord = vec2(ex_TexCoord0.x*0.015779, ex_TexCoord0.y*0.014109);
	vec4 texturefilter = texture(sFilter, filtercoord);
	vec4 texturecolor;
	float alpha = texturefilter.a;
	texturefilter.rgb*=alpha;
	texturecolor= texturecolor1*texturefilter.b+texturecolor3*texturefilter.g+texturecolor2*texturefilter.r+texturecolor4*(1-texturefilter.a);


	if(texture(shadowmap, projected).r < depth ){
		//in shadow
		vec4 color = texturecolor*0.2 + texturecolor*diffuseIntensity + vec4(1.0, 1.0,1.0, 1.0)*specular;
		if(diffuseIntensity> 0.1){
			color*=0.4;
		}
		out_Color = color;
	}
	else{
		out_Color = texturecolor*0.2 + texturecolor*diffuseIntensity + vec4(1.0, 1.0,1.0, 1.0)*specular;
	}
/*
	vec4 color = texturecolor*0.2 + texturecolor*diffuseIntensity + vec4(1.0, 1.0,1.0, 1.0)*specular;
		
	out_Color = color*VSM(depth, projected);
*/
	
}
